/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.controller;

import cz.muni.csirt.analyza.util.config.ProxyProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;


/**
 * General controller for both generic objects and links user property issues.
 **/
@RestController
public class AbstractObjectController {

    private ProxyProperties proxyProperties = new ProxyProperties();
    private String warehouseLocation = proxyProperties.getLocalWarehouseUrl();
    RestTemplate templ = new RestTemplate();

    @Autowired
    public AbstractObjectController() { }

    @ResponseBody
    @RequestMapping(value = {"/generic-objects/{uuid}"}, method = RequestMethod.PUT)
    public ResponseEntity<Object> updateGenericObjectsUserProperty(@PathVariable("uuid") String uuid,
                                                                   HttpServletRequest httpRequest,
                                                                   HttpMethod method,
                                                                   HttpEntity<Object> httpEntity) {
        ResponseEntity<Object> resp;
        try {
            String url = warehouseLocation + "generic-objects/" + uuid + "?" + httpRequest.getQueryString();
            url = URLDecoder.decode(url, StandardCharsets.UTF_8.toString());
            resp = templ.exchange(url, method, httpEntity, Object.class);
        } catch (RuntimeException | UnsupportedEncodingException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
        return resp;
    }

    @ResponseBody
    @RequestMapping(value = {"/links/{uuid}"}, method = RequestMethod.PUT)
    public ResponseEntity<Object> updateLinkUserProperty(@PathVariable("uuid") String uuid,
                                                         HttpServletRequest httpRequest,
                                                         HttpMethod method,
                                                         HttpEntity<Object> httpEntity) {
        ResponseEntity<Object> resp;
        try {
            String url = warehouseLocation + "links/" + uuid + "?" + httpRequest.getQueryString();
            url = URLDecoder.decode(url, StandardCharsets.UTF_8.toString());
            resp = templ.exchange(url, method, httpEntity, Object.class);
        } catch (RuntimeException | UnsupportedEncodingException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
        return resp;
    }
}
