/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.controller;

import cz.muni.csirt.analyza.util.config.ProxyProperties;
import org.apache.commons.io.IOUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;


/**
 * Controller for work with DataFiles.
 */
@RestController
@RequestMapping(value = "/generic-objects/data-files")
public class DataFileController {
    private ProxyProperties proxyProperties = new ProxyProperties();
    private String warehouseLocation = proxyProperties.getLocalWarehouseUrl() + "generic-objects/data-files";

    RestTemplate templ = new RestTemplate();

    @Autowired
    public DataFileController(){ }

    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, produces = "application/json")
    public ResponseEntity<String> uploadAndCreateFile(@RequestParam("file")MultipartFile file, HttpServletRequest httpRequest) {
        HttpResponse response;
        String body;
        try {
            String url = warehouseLocation + "?" + httpRequest.getQueryString();
            HttpPost post = new HttpPost(url);
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            builder.addBinaryBody("file", file.getInputStream(), ContentType.DEFAULT_BINARY, file.getOriginalFilename());
            post.setEntity(builder.build());
            response = HttpClients.createDefault().execute(post);
            body = IOUtils.toString(response.getEntity().getContent(), "UTF-8");
        } catch (IOException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(body);
    }

    @ResponseBody
    @RequestMapping(value = "/{uuid}", method = RequestMethod.GET)
    public ResponseEntity<Resource> serveFile(@PathVariable("uuid") String uuid, HttpMethod method, HttpEntity<Resource> httpEntity) {
        ResponseEntity<Resource> resp;
        String url = warehouseLocation + "/" + uuid;
        try {
            resp = templ.exchange(url, method, httpEntity, Resource.class);
        } catch (RestClientException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
        return resp;
    }

    @ResponseBody
    @RequestMapping(value = "/{uuid}", method = RequestMethod.PUT)
    public ResponseEntity<String> uploadFile( @RequestParam("file")MultipartFile file, @PathVariable("uuid") String uuid) {
        HttpResponse response;
        String body;
        try {
            HttpPut put = new HttpPut(warehouseLocation + "/" + uuid);
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            builder.addBinaryBody("file", file.getInputStream(), ContentType.DEFAULT_BINARY, file.getOriginalFilename());
            put.setEntity(builder.build());
            response = HttpClients.createDefault().execute(put);
            body = IOUtils.toString(response.getEntity().getContent(), "UTF-8");
        } catch (IOException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
            return ResponseEntity.ok(body);
    }

    @RequestMapping(value = "/{uuid}", method = RequestMethod.DELETE)
    public ResponseEntity<String> deleteFile(@PathVariable("uuid") String uuid, HttpMethod method, HttpEntity<String> httpEntity){
        ResponseEntity<String> resp;
        String url = warehouseLocation + "/" + uuid;
        try {
            resp = templ.exchange(url, method, httpEntity, String.class);
        } catch (RestClientException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
        return resp;
    }

    @ResponseBody
    @RequestMapping(value = "/local", method = RequestMethod.POST, produces = "application/json")
    public ResponseEntity<String> uploadAndCreateFileLocal(HttpServletRequest httpRequest,
                                                           HttpEntity<String> httpEntity) {
        ResponseEntity<String> resp;
        try {
            String url = warehouseLocation + "/local?" + httpRequest.getQueryString();
            url = URLDecoder.decode(url, StandardCharsets.UTF_8.toString());
            resp = templ.exchange(url, HttpMethod.POST, httpEntity, String.class);
        } catch (UnsupportedEncodingException | RestClientException e) {
            System.err.println("Problem while resending.");
            return ResponseEntity.notFound().build();
        }
        return resp;
    }
}
