/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import cz.muni.csirt.analyza.json.AbstractObjectDeserializer;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Objects;
import java.util.UUID;

/**
 * Class that holds AbstractObject and its attributes.
 */
@JsonDeserialize(using = AbstractObjectDeserializer.class)
public class AbstractObject
{
    @JsonIgnore
    private Long id;

    private UUID uuid;

    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private LocalDateTime created;

    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private LocalDateTime modified;

    private String caseId;

    private Collection<UserProperty> properties = new ArrayList<>();

    protected AbstractObject() {

        this.uuid = UUID.randomUUID();
        LocalDateTime now = LocalDateTime.now();
        this.created = now;
        this.modified = now;
        this.caseId = null;
    }

    protected AbstractObject(AbstractObject abstractObject) {
        this.id = abstractObject.getId();
        this.uuid = abstractObject.getUuid();
        this.created = abstractObject.getCreated();
        this.modified = abstractObject.getModified();
        this.caseId = abstractObject.getCaseId();
        this.properties = abstractObject.getProperties();
    }

    protected AbstractObject(UUID uuid, LocalDateTime created, LocalDateTime modified, String caseId) {
        this.uuid = uuid;
        this.created = created;
        this.modified = modified;
        this.caseId = caseId;
    }

    public Long getId() {
        return id;
    }

    public AbstractObject setId(Long id) {
        this.id = id;
        return this;
    }

    public UUID getUuid() {
        return uuid;
    }

    public AbstractObject setUuid(UUID uuid) {
        this.uuid = uuid;
        return this;
    }

    public LocalDateTime getCreated() {
        return created;
    }

    public LocalDateTime getModified() {
        return modified;
    }

    public String getCaseId() {
        return caseId;
    }

    public Collection<UserProperty> getProperties() {
        return properties;
    }

    public AbstractObject setProperties(Collection<UserProperty> properties) {
        this.properties = properties;
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof AbstractObject)) return false;
        AbstractObject that = (AbstractObject) o;

        return  Objects.equals(uuid, that.uuid) &&
                Objects.equals(created, that.created) &&
                Objects.equals(modified, that.modified) &&
                Objects.equals(caseId, that.caseId) &&
                Objects.equals(properties, that.properties);
    }

    @Override
    public int hashCode() {
        return Objects.hash(uuid, created, modified, caseId, properties);
    }

    @Override
    public String toString() {
        return "AbstractObject{" +
                "id=" + getId() +
                ", uuid=" + getUuid() +
                ", created=" + getCreated() +
                ", modified=" + getModified() +
                ", caseId='" + getCaseId() + '\'' +
                ", properties=" + getProperties() +
                '}';
    }
}
