/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.entity;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import cz.muni.csirt.analyza.json.DataFileDeserializer;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Locale;
import java.util.Objects;
import java.util.UUID;


/**
 * Class that holds DataFile and its attributes.
 */
@JsonDeserialize(using = DataFileDeserializer.class)
@JsonPropertyOrder({"uuid", "created", "modified", "name", "type", "mimeType", "fileLocation", "size", "caseId", "properties"})
public class DataFile extends GenericObject {

    private String mimeType;

    private String fileLocation;

    private Long size;

    public DataFile() {}

    public DataFile(String name, ObjectType type, String mimeType, String fileLocation, Long size) {
        super(name, type);
        this.mimeType = mimeType;
        this.fileLocation = fileLocation;
        this.size = size;
    }

    public DataFile(GenericObject genericObject, String mimeType, String fileLocation, Long size) {
        super(genericObject);
        this.mimeType = mimeType;
        this.fileLocation = fileLocation;
        this.size = size;
    }

    public DataFile(UUID uuid, LocalDateTime created, LocalDateTime modified, String caseId, String name,
                    ObjectType type,  String mimeType, String fileLocation, Long size) {
        super(uuid, created, modified, caseId, name, type);
        this.mimeType = mimeType;
        this.fileLocation = fileLocation;
        this.size = size;
    }

    public String getMimeType() {
        return mimeType;
    }

    public DataFile setMimeType(String mimeType) {
        this.mimeType = mimeType;
        return this;
    }

    public String getFileLocation() {
        return fileLocation;
    }

    public DataFile setFileLocation(String fileLocation) {
        this.fileLocation = fileLocation;
        return this;
    }

    public Long getSize() {
        return size;
    }

    public DataFile setSize(Long size) {
        this.size = size;
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof DataFile)) return false;
        if (!super.equals(o)) return false;
        DataFile dataFile = (DataFile) o;
        return Objects.equals(mimeType, dataFile.mimeType) &&
                Objects.equals(fileLocation, dataFile.fileLocation) &&
                Objects.equals(size, dataFile.size);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), mimeType, fileLocation, size);
    }

    @Override
    public String toString() {
        ObjectWriter ow = new ObjectMapper().writerWithDefaultPrettyPrinter().with(Locale.forLanguageTag("yyyy-MM-dd HH:mm:ss"));
        ObjectMapper objectMapper = new ObjectMapper();
        String json_pretty = "";
        try {
            json_pretty = ow.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        LocalDateTime.parse(json_pretty, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        JsonNode jsonNode = null;
        try {
            jsonNode = objectMapper.readTree(json_pretty);
        } catch (IOException e) {
            e.printStackTrace();
        }
        json_pretty.replace("created", LocalDateTime.parse(this.getCreated().toString(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).toString()); //todo remove?

        return json_pretty;
    }
}
