/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.annotation.JsonUnwrapped;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import cz.muni.csirt.analyza.json.GenericObjectDeserializer;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Objects;
import java.util.UUID;

/**
 * Class that holds GenericObject and its attributes.
 */
@JsonDeserialize(using = GenericObjectDeserializer.class)
@JsonPropertyOrder({"uuid", "created", "modified", "name", "type", "caseId", "properties"})
public class GenericObject extends AbstractObject {

    private String name;

    @JsonIgnore
    private Collection<Link> leftLinks = new ArrayList<>();

    @JsonIgnore
    private Collection<Link> rightLinks = new ArrayList<>();

    @JsonUnwrapped
    private ObjectType type;

    public GenericObject() {}

    public GenericObject(String name, ObjectType type) {
        this.name = name;
        this.type = type;
    }

    public GenericObject(GenericObject genericObject) {
        super(genericObject);
        this.name = genericObject.getName();
        this.type = genericObject.getType();
        this.leftLinks = genericObject.getLeftLinks();
        this.rightLinks = genericObject.getRightLinks();
    }

    public GenericObject(UUID uuid, LocalDateTime created, LocalDateTime modified, String caseId, String name, ObjectType type) {
        super(uuid, created, modified, caseId);
        this.name = name;
        this.type = type;
    }

    public String getName() {
        return name;
    }

    public ObjectType getType() {
        return type;
    }

    public Collection<Link> getLeftLinks() {
        return leftLinks;
    }

    public Collection<Link> getRightLinks() {
        return rightLinks;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof GenericObject)) return false;
        GenericObject that = (GenericObject) o;
        return Objects.equals(name, that.name) &&
                Objects.equals(type, that.type);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, type);
    }


    @Override
    public String toString() {
        return "{\"uuid\":\"" + getUuid() +
                        "\",\"created\":\"" + getCreated() +
                        "\",\"modified\":\"" + getModified() +
                        "\",\"caseId\":\"" + getCaseId() +
                        "\",\"properties\":"  + getProperties() +
                        ",\"name\":\"" + getName() +
                        "\",\"type\":" + getType() + "}";
    }
}
