/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.annotation.JsonUnwrapped;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import cz.muni.csirt.analyza.json.LinkDeserializer;
import java.time.LocalDateTime;
import java.util.Objects;
import java.util.UUID;

/**
 * Class that holds Link and its attributes.
 */
@JsonDeserialize(using = LinkDeserializer.class)
@JsonPropertyOrder({"uuid", "created", "modified", "left", "right", "type", "oriented", "caseId", "properties"})
public class Link extends AbstractObject {

    @JsonIgnore
    private GenericObject left;

    @JsonIgnore
    private GenericObject right;

    private boolean oriented;

    @JsonUnwrapped
    private LinkType type;

    public Link() {}

    public Link(GenericObject left, GenericObject right, LinkType type, boolean oriented) {
        this.left = left;
        this.right = right;
        this.type = type;
        this.oriented = oriented;
    }

    public Link(Link link) {
        super(link);
        this.left = link.getLeft();
        this.right = link.getRight();
        this.oriented = link.isOriented();
        this.type = link.getType();
    }

    public Link(UUID uuid, LocalDateTime created, LocalDateTime modified, String caseId, GenericObject left, GenericObject right, boolean oriented, LinkType type) {
        super(uuid, created, modified, caseId);
        this.left = left;
        this.right = right;
        this.type = type;
        this.oriented = oriented;
    }

    public GenericObject getLeft() {
        return left;
    }

    public GenericObject getRight() {
        return right;
    }

    public LinkType getType() {
        return type;
    }

    public boolean isOriented() {
        return oriented;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Link)) return false;
        Link link = (Link) o;
        return oriented == link.oriented &&
                Objects.equals(left, link.left) &&
                Objects.equals(right, link.right) &&
                Objects.equals(type, link.type);
    }

    @Override
    public int hashCode() {
        return Objects.hash(left, right, oriented, type);
    }

    @Override
    public String toString() {
        return
                "{\"id\":\"" + getId() +
                        "\",\"uuid\":\"" + getUuid() +
                        "\",\"created\":\"" + getCreated() +
                        "\",\"modified\":\"" + getModified() +
                        "\",\"caseId\":\"" + getCaseId() +
                        "\",\"properties\":"  + getProperties() +
                        ",\"left\":" + left +
                        ",\"right\":" + right +
                        ",\"oriented\":\"" + oriented +
                        "\",\"type\":" + type + "}";
    }
}
