/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import cz.muni.csirt.analyza.json.UserPropertyDeserializer;
import java.util.Objects;
import java.util.UUID;

/**
 * Class that holds UserProperty and its attributes.
 */
@JsonDeserialize(using = UserPropertyDeserializer.class)
public class UserProperty {

    @JsonIgnore
    private Long id;

    private String propertyKey;

    @JsonInclude(JsonInclude.Include.NON_NULL)
    private Long valueLong;

    @JsonInclude(JsonInclude.Include.NON_NULL)
    private Double valueDouble;

    @JsonInclude(JsonInclude.Include.NON_NULL)
    private String valueString;

    @JsonIgnore
    private AbstractObject parent;

    @JsonIgnore
    private UUID parentUuid;

    public UserProperty() {}

    public UserProperty(UserProperty userProperty) {
        this.id = userProperty.getId();
        this.parent = userProperty.getParent();
        this.parentUuid = parent.getUuid();
        this.propertyKey = userProperty.getPropertyKey();
        this.valueLong = userProperty.getValueLong();
        this.valueDouble = userProperty.getValueDouble();
        this.valueString = userProperty.getValueString();
    }

    public UserProperty(AbstractObject parentObject, String propertyKey, String valueString) {
        this.parent = parentObject;
        this.parentUuid = parentObject.getUuid();
        this.propertyKey = propertyKey;
        this.valueString = valueString;
        this.valueLong = null;
        this.valueDouble = null;
    }

    public UserProperty(String propertyKey, String valueString) {
        this.propertyKey = propertyKey;
        this.valueString = valueString;
        this.valueLong = null;
        this.valueDouble = null;
    }

    public UserProperty(AbstractObject parentObject, String propertyKey, Double valueDouble) {
        this.parent = parentObject;
        this.parentUuid = parentObject.getUuid();
        this.propertyKey = propertyKey;
        this.valueDouble = valueDouble;
        this.valueLong = null;
        this.valueString = null;
    }

    public UserProperty(String propertyKey, Double valueDouble) {
        this.propertyKey = propertyKey;
        this.valueDouble = valueDouble;
        this.valueLong = null;
        this.valueString = null;
    }

    public UserProperty(AbstractObject parentObject, String propertyKey, Long valueLong) {
        this.parent = parentObject;
        this.parentUuid = parentObject.getUuid();
        this.propertyKey = propertyKey;
        this.valueLong = valueLong;
        this.valueString = null;
        this.valueDouble = null;
    }

    public UserProperty(String propertyKey, Long valueLong) {
        this.propertyKey = propertyKey;
        this.valueLong = valueLong;
        this.valueString = null;
        this.valueDouble = null;
    }

    public Long getId() {
        return id;
    }

    public UserProperty setId(Long id) {
        this.id = id;
        return this;
    }

    public AbstractObject getParent() {
        return parent;
    }

    public UserProperty setParentUuid(UUID parentUuid) {
        this.parentUuid = parentUuid;
        return this;
    }

    public String getPropertyKey() {
        return propertyKey;
    }

    public UserProperty setPropertyKey(String propertyKey) {
        this.propertyKey = propertyKey;
        return this;
    }

    public Long getValueLong() {
        return valueLong;
    }

    public UserProperty setValueLong(Long valueLong) {
        this.valueLong = valueLong;
        return this;
    }

    public Double getValueDouble() {
        return valueDouble;
    }

    public UserProperty setValueDouble(Double valueDouble) {
        this.valueDouble = valueDouble;
        return this;
    }

    public String getValueString() {
        return valueString;
    }

    public UserProperty setValueString(String valueString) {
        this.valueString = valueString;
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof UserProperty)) return false;
        UserProperty that = (UserProperty) o;
        return Objects.equals(id, that.id) &&
                Objects.equals(parentUuid, that.parentUuid) &&
                Objects.equals(propertyKey, that.propertyKey) &&
                Objects.equals(valueLong, that.valueLong) &&
                Objects.equals(valueDouble, that.valueDouble) &&
                Objects.equals(valueString, that.valueString);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, parentUuid, propertyKey, valueLong, valueDouble, valueString);
    }

    @Override
    public String toString() {
        return "{\"id\":\"" + id +
                "\",\"parentUuid\":\"" + parentUuid +
                "\",\"propertyKey\":\"" + propertyKey +
                "\",\"value\":\"" + valueString + "\"}";
    }
}
