/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.json;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import cz.muni.csirt.analyza.entity.AbstractObject;
import cz.muni.csirt.analyza.entity.GenericObject;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;


public class AbstractObjectDeserializer extends StdDeserializer<AbstractObject> {

    protected AbstractObjectDeserializer() {
        this(null);
    }

    protected AbstractObjectDeserializer(Class<?> vc) {
        super(vc);
    }

    @Override
    public AbstractObject deserialize(JsonParser jp, DeserializationContext deserializationContext) throws IOException, JsonProcessingException {
        JsonNode node = jp.getCodec().readTree(jp);

        Long id = null;
        if (node.get("id") != null) {
            id = node.get("id").asLong();
        }
        UUID uuid = UUID.fromString(node.get("uuid").asText());
        LocalDateTime created, modified;
        List<LocalDateTime> localDateTimes = BasicLocalDateTimeDeserializer.parseLocalDateTimes(node);
        LocalDateTime crea_t = LocalDateTime.parse(node.get("created").asText());
        created = localDateTimes.get(0);
        modified = localDateTimes.get(1);
        String caseId = node.get("caseId").asText();

        // todo user properties

        return new GenericObject(uuid, created, modified, caseId, null, null).setId(id);
    }

}
