/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.json;

import com.fasterxml.jackson.databind.JsonNode;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

/**
 * @author David Brilla*xbrilla*469054
 */
public class BasicLocalDateTimeDeserializer {

    //{"nano":990823000,"year":2019,"monthValue":11,"dayOfMonth":19,"hour":14,"minute":38,"second":34,"dayOfWeek":"TUESDAY","dayOfYear":323,"month":"NOVEMBER","chronology":{"calendarType":"iso8601","id":"ISO"}}

    private static LocalDateTime deserialize(JsonNode node) {
        int year = node.get("year").asInt();
        int month = node.get("monthValue").asInt();
        int day = node.get("dayOfMonth").asInt();
        int hour = node.get("hour").asInt();
        int min = node.get("minute").asInt();
        int sec = node.get("second").asInt();

        return LocalDateTime.of(year, month, day, hour, min, sec);
    }

    private static LocalDateTime deserializeTimestamp(String localDateTime) {
        String[] splits = localDateTime.split(" ");
        String[] dates = splits[0].split("-");
        String[] times = splits[1].split(":");
        return LocalDateTime.of(
                Integer.parseInt(dates[0]),
                Integer.parseInt(dates[1]),
                Integer.parseInt(dates[2]),
                Integer.parseInt(times[0]),
                Integer.parseInt(times[1]),
                Integer.parseInt(times[2]));
    }

    public static List<LocalDateTime> parseLocalDateTimes(JsonNode node) {
        LocalDateTime created, modified;
        if (node.get("created").asText().isEmpty()) {
            created = deserialize(node.get("created"));
            modified = deserialize(node.get("modified"));
        } else {
            created = deserializeTimestamp(node.get("created").asText());
            modified = deserializeTimestamp(node.get("modified").asText());
        }
        return Arrays.asList(created, modified);
    }
}
