/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.json;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.fasterxml.jackson.databind.type.TypeFactory;
import cz.muni.csirt.analyza.entity.DataFile;
import cz.muni.csirt.analyza.entity.ObjectType;
import cz.muni.csirt.analyza.entity.UserProperty;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

public class DataFileDeserializer extends StdDeserializer<DataFile> {

    public DataFileDeserializer() {
        this(null);
    }

    public DataFileDeserializer(Class<?> vc) {
        super(vc);
    }

    @Override
    public DataFile deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException {

        JsonNode node = jp.getCodec().readTree(jp);

        Long id = null;
        JsonNode sid = node.get("id");
        if (sid != null && !sid.toString().equals("null")) {
            id = node.get("id").asLong();
        }
        String mimeType = node.get("mimeType").asText();
        String fileLocation = node.get("fileLocation").asText();
        Long size = node.get("size").asLong();

        UUID uuid = UUID.fromString(node.get("uuid").asText());
        LocalDateTime created, modified;
        List<LocalDateTime> localDateTimes = BasicLocalDateTimeDeserializer.parseLocalDateTimes(node);
        created = localDateTimes.get(0);
        modified = localDateTimes.get(1);
        CollectionType typeReference = TypeFactory.defaultInstance().constructCollectionType(Collection.class, UserProperty.class);
        Collection<UserProperty> properties = new ObjectMapper().readValue(node.get("properties").toString(), typeReference);
        for (UserProperty up : properties) {
            up.setParentUuid(uuid);
        }
        String caseId = node.get("caseId").asText();
        String name = node.get("name").asText();
        ObjectType type = new ObjectType(node.get("type").asText());


        DataFile result = new DataFile(uuid, created, modified, caseId, name, type, mimeType, fileLocation, size);
        result.setId(id).setProperties(properties);
        return result;
    }
}
