/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.json;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.fasterxml.jackson.databind.type.TypeFactory;
import cz.muni.csirt.analyza.entity.*;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

/**
 * JSON Deserializer for Link
 */
public class LinkDeserializer extends StdDeserializer<Link> {


    public LinkDeserializer() {
        this(null);
    }

    public LinkDeserializer(Class<?> vc) {
        super(vc);
    }

    @Override
    public Link deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException {

        JsonNode node = jp.getCodec().readTree(jp);

        Long id = null;
        if (node.get("id") != null) {
            id = node.get("id").asLong();
        }
        UUID uuid = UUID.fromString(node.get("uuid").asText());
        LocalDateTime created, modified;
        List<LocalDateTime> localDateTimes = BasicLocalDateTimeDeserializer.parseLocalDateTimes(node);
        created = localDateTimes.get(0);
        modified = localDateTimes.get(1);
        CollectionType typeReference = TypeFactory.defaultInstance().constructCollectionType(Collection.class, UserProperty.class);
        Collection<UserProperty> properties = new ObjectMapper().readValue(node.get("properties").toString(), typeReference);
        for (UserProperty up : properties) {
            up.setParentUuid(uuid);
        }
        String caseId = node.get("caseId").asText();

        GenericObject left, right;
        try {
            left = new ObjectMapper().readValue(node.get("left").toString(), DataFile.class);
        } catch (Exception e) {
            try {
                left = new ObjectMapper().readValue(node.get("left").toString(), GenericObject.class);
            } catch (Exception ex) {
                left = (GenericObject) new GenericObject().setUuid(UUID.fromString(node.get("left").asText()));
            }
        }
        try {
            right = new ObjectMapper().readValue(node.get("right").toString(), DataFile.class);
        } catch (Exception e) {
            try {
                right = new ObjectMapper().readValue(node.get("right").toString(), GenericObject.class);
            } catch (Exception ex) {
                right = (GenericObject) new GenericObject().setUuid(UUID.fromString(node.get("right").asText()));
            }
        }
        boolean oriented = node.get("oriented").asBoolean();
        LinkType type = new LinkType(node.get("type").asText());

        Link result = new Link(uuid, created, modified, caseId, left, right, oriented, type);
        result.setId(id).setProperties(properties);
        return result;
    }
}
