/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.json;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import cz.muni.csirt.analyza.entity.UserProperty;

import java.io.IOException;
import java.util.UUID;

/**
 * JSON Deserializer for User Property
 */
public class UserPropertyDeserializer extends StdDeserializer<UserProperty> {


    private static final String PARENT_UUID = "parentUuid";
    private static final String PROPERTY_KEY = "propertyKey";
    private static final String VALUE_LONG = "valueLong";
    private static final String VALUE_DOUBLE = "valueDouble";
    private static final String VALUE_STRING = "valueString";

    public UserPropertyDeserializer() {
        this(null);
    }

    protected UserPropertyDeserializer(Class<?> vc) {
        super(vc);
    }

    @Override
    public UserProperty deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException {

        JsonNode node = jp.getCodec().readTree(jp);

        UserProperty result = new UserProperty();

        if (node.get("id") != null)
            result.setId(node.get("id").asLong());

        if (node.get(PARENT_UUID) != null && !node.get(PARENT_UUID).toString().equals("null")) {
            result.setParentUuid(UUID.fromString(node.get(PARENT_UUID).asText()));
        }
        if(node.get(PROPERTY_KEY) != null)
            result.setPropertyKey(node.get(PROPERTY_KEY).asText());

        if (node.get(VALUE_LONG) != null && !node.get(VALUE_LONG).toString().equals("null"))
            return result.setValueLong(node.get(VALUE_LONG).asLong());

        if (node.get(VALUE_DOUBLE) != null && !node.get(VALUE_DOUBLE).toString().equals("null"))
            return result.setValueDouble(node.get(VALUE_DOUBLE).asDouble());

        return result.setValueString(node.get(VALUE_STRING).asText());
    }
}
