/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.csirt.analyza.entity.DataFile;
import cz.muni.csirt.analyza.entity.GenericObject;
import cz.muni.csirt.analyza.entity.UserProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * Implementation of GenericObjectService interface.
 */
@Service
public class GenericObjectServiceImpl implements GenericObjectService {

    @Autowired
    public GenericObjectServiceImpl() {}

    public Collection<GenericObject> getFullHierarchy(String localW, ArrayList<String> hierarchyW, String mode) {
        if (mode.equals("A")) {
            ///// mode "A"
            hierarchyW.add(localW);
            return getFullByTime(hierarchyW);
        } else {
            ///// mode "B"
            return getFullByLocal(localW, hierarchyW);
        }
    }

    public Collection<GenericObject> getFullByTime(ArrayList<String> hierarchyW) {
        Map<String, GenericObject> mapG = new TreeMap<>();
        ObjectMapper objectMapper = new ObjectMapper();

        for (String json : hierarchyW) {
            try{
                JsonNode json_fin = objectMapper.readTree(json);
                Iterator<JsonNode> iter = json_fin.elements();
                while (iter.hasNext()) {
                    ArrayList<UserProperty> propYounger;
                    ArrayList<UserProperty> propOlder;
                    GenericObject g;
                    Object a = iter.next();
                    try {
                        g = objectMapper.convertValue(a, DataFile.class);
                    } catch (Exception e) {
                        g = objectMapper.convertValue(a, GenericObject.class);
                    }
                    String currUUID = g.getUuid().toString();
                    if (mapG.containsKey(currUUID)) {
                        if (mapG.get(currUUID).getModified().isBefore(g.getModified())) {
                            propYounger = new ArrayList<>(g.getProperties());
                            propOlder = new ArrayList<>(mapG.get(currUUID).getProperties());
                            propYounger = mergeProperties(propYounger, propOlder);
                            g.setProperties(new ArrayList<>(propYounger));
                            mapG.put(currUUID, g);
                        } else {
                            propYounger = new ArrayList<>(mapG.get(currUUID).getProperties());
                            propOlder = new ArrayList<>(g.getProperties());
                            propYounger = mergeProperties(propYounger, propOlder);
                            mapG.get(currUUID).setProperties(new ArrayList<>(propYounger));
                            propYounger.clear();
                        }
                    } else {
                        mapG.put(currUUID, g);
                    }
                }
           } catch (IOException e) {
                System.err.println("Generic-objects mode A: Problem with reading json.");
            }
        }
       return mapG.values();
    }

    public Collection<GenericObject> getFullByLocal(String localW, ArrayList<String> hierarchyW){
        Map<String, GenericObject> mapG = stringToGenericObjectsMap(localW);
        ObjectMapper objectMapper = new ObjectMapper();

        for ( String json : hierarchyW) {
            GenericObject g;
            try {
                JsonNode json_fin = objectMapper.readTree(json);
                Iterator<JsonNode> iter = json_fin.elements();
                while (iter.hasNext()) {
                    Object a = iter.next();
                    try {
                        g = objectMapper.convertValue(a, DataFile.class);
                    } catch (Exception e) {
                        g = objectMapper.convertValue(a, GenericObject.class);
                    }
                        String currUUID = g.getUuid().toString();
                        if (mapG.containsKey(currUUID)) {
                            ArrayList<UserProperty> propLocal = new ArrayList<>(mapG.get(currUUID).getProperties());
                            ArrayList<UserProperty> propRemote = new ArrayList<>(g.getProperties());
                            propLocal = mergeProperties(propLocal, propRemote);
                            mapG.get(currUUID).setProperties(new ArrayList<>(propLocal));
                            propLocal.clear();
                        }
                        mapG.put(currUUID, g);
                }
            } catch (IOException e) {
                System.err.println("Generic-objects mode B: Problem with reading json.");
            }
        }
        return mapG.values();
    }

    public ArrayList<UserProperty> mergeProperties(ArrayList<UserProperty> primary, ArrayList<UserProperty> secondary) {
        for (UserProperty prop: secondary) {
            if (!primary.contains(prop)) {
                primary.add(prop);
            }
        }
        return primary;
    }

    public Map<String, GenericObject> stringToGenericObjectsMap(String localAnswer) {
        Map<String, GenericObject> mapG = new TreeMap<>();
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode json_fin = objectMapper.readTree(localAnswer);
            Iterator<JsonNode> iter = json_fin.elements();
            while (iter.hasNext()) {
                Object a = iter.next();
                GenericObject g;
                try {
                    g = objectMapper.convertValue(a, DataFile.class);
                } catch (Exception e) {
                    g = objectMapper.convertValue(a, GenericObject.class);
                }
                mapG.put(g.getUuid().toString(), g);
            }
        } catch (RestClientException | IOException e) {
            System.err.println(e.toString());
        }
        return mapG;
    }
}
