/*
 * Copyright 2020 Masarykova univerzita
 *
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.muni.csirt.analyza.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.csirt.analyza.entity.Link;
import cz.muni.csirt.analyza.entity.UserProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * Implementation of LinkService interface.
 */
@Service
public class LinkServiceImpl implements LinkService {

    @Autowired
    public LinkServiceImpl() { }

    @Override
    public Collection<Link> getFullHierarchy(ArrayList<String> hierarchyW, String mode) {
        Collection<Link> response;
        if (mode.equals("A")) {
            // mode "A"
            response = getFullByTime(hierarchyW);
        } else {
            // mode "B"
            response = getFullByLocal(hierarchyW);
        }
        return response;
    }

    @Override
    public Collection<Link> getFullByTime(ArrayList<String> hierarchyW) {
        Map<String, Link> mapL = new TreeMap<>();
        ObjectMapper objectMapper = new ObjectMapper();

        for ( String json : hierarchyW) {
            JsonNode json_fin;
            try {
                json_fin = objectMapper.readTree(json);
            } catch (IOException e) {
                System.err.println("LinkService mode A: Problem with reading json.");
                continue;
            }
            Iterator<JsonNode> iter = json_fin.elements();
            while (iter.hasNext()) {
                ArrayList<UserProperty> propYounger;
                ArrayList<UserProperty> propOlder;
                Link l = new Link();
                Object a = iter.next();
                try {
                    l = objectMapper.readValue(a.toString(), Link.class);
                } catch (IOException e) {
                    System.err.println("LinkService mode A: Problem with reading json.");
                }
                String currUUID = l.getUuid().toString();
                if (mapL.containsKey(currUUID)) {
                    if (mapL.get(currUUID).getModified().isBefore(l.getModified())) {
                        propYounger = new ArrayList<>(l.getProperties());
                        propOlder = new ArrayList<>(mapL.get(currUUID).getProperties());
                        propYounger = mergeProperties(propYounger, propOlder);
                        l.setProperties(new ArrayList<>(propYounger));
                        mapL.put(currUUID, l);
                    } else {
                        propYounger = new ArrayList<>(mapL.get(currUUID).getProperties());
                        propOlder = new ArrayList<>(l.getProperties());
                        propYounger = mergeProperties(propYounger, propOlder);
                        mapL.get(currUUID).setProperties(new ArrayList<>(propYounger));
                        propYounger.clear();
                    }
                } else {
                    mapL.put(currUUID, l);
                }
            }
        }
        return mapL.values();
    }

    @Override
    public Collection<Link> getFullByLocal(ArrayList<String> hierarchyW) {
        Map<String, Link> mapL = new TreeMap<>();
        ObjectMapper objectMapper = new ObjectMapper();

        for ( String json : hierarchyW) {
            JsonNode json_fin;
            try {
                json_fin = objectMapper.readTree(json);
            } catch (IOException e) {
                System.err.println("LinkService mode B: Problem with reading json.");
                continue;
            }
            Iterator<JsonNode> iter = json_fin.elements();
            while (iter.hasNext()) {
                ArrayList<UserProperty> propLocal;
                ArrayList<UserProperty> propRemote;
                Link l = new Link();
                Object a = iter.next();
                try {
                    l = objectMapper.readValue(a.toString(), Link.class);
                } catch (IOException e) {
                    System.err.println("LinkService mode B: Problem with reading json.");
                }
                String currUUID = l.getUuid().toString();
                if (mapL.containsKey(currUUID)) {
                    propLocal = new ArrayList<>(mapL.get(currUUID).getProperties());
                    propRemote = new ArrayList<>(l.getProperties());
                    propLocal = mergeProperties(propLocal, propRemote);
                    mapL.get(currUUID).setProperties(new ArrayList<>(propLocal));
                    propLocal.clear();
                } else {
                    mapL.put(currUUID, l);
                }
            }
        }
        return mapL.values();
    }

    public ArrayList<UserProperty> mergeProperties(ArrayList<UserProperty> primary, ArrayList<UserProperty> secondary) {
        for (UserProperty prop: secondary) {
            if (!primary.contains(prop)) {
                primary.add(prop);
            }
        }
        return primary;
    }
}
